bison-patches
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

Re: doc fixes for bison (bison dir/foo.y, etc.)


From: Paul Eggert
Subject: Re: doc fixes for bison (bison dir/foo.y, etc.)
Date: Thu, 15 Sep 2005 06:32:51 -0700
User-agent: Gnus/5.1007 (Gnus v5.10.7) Emacs/21.4 (gnu/linux)

Akim Demaille <address@hidden> writes:

> I'm confused with the fact that
>
> /tmp % bison -S lalr1.cc bar/foo.y -o bar/foo.cc                 nostromo 
> 13:40
> /tmp % ls -l bar                                                 nostromo 
> 13:40
> total 24
> -rw-r--r--  1 akim lrde 17801 2005-09-15 13:40 foo.cc
> -rw-r--r--  1 akim lrde    15 2005-09-15 13:39 foo.y
> /tmp % bison -S lalr1.cc bar/foo.y -o bar/foo.cc -d -v           nostromo 
> 13:41
> /tmp % ls -l bar                                                 nostromo 
> 13:41
> total 40
> -rw-r--r--  1 akim lrde 17823 2005-09-15 13:41 foo.cc
> -rw-r--r--  1 akim lrde  8506 2005-09-15 13:41 foo.hh
> -rw-r--r--  1 akim lrde   582 2005-09-15 13:41 foo.output
> -rw-r--r--  1 akim lrde    15 2005-09-15 13:39 foo.y
> /tmp % ls position.hh location.hh stack.hh                       nostromo 
> 13:41
> location.hh  position.hh  stack.hh
>
> These are wrong.

Yes, that's a known bug.  Oleg Smolsky has proposed a patch
<http://lists.gnu.org/archive/html/bug-bison/2005-09/msg00002.html>
but it wasn't general enough to suit my taste and also I'm leery of
applying it just before 2.1 comes out.

Here is the direction I was headed to fix the problem.  I haven't
compiled or tested this patch, and it does not work.  Some more
changes to the skeleton would be needed.

Index: src/files.h
===================================================================
RCS file: /cvsroot/bison/bison/src/files.h,v
retrieving revision 1.39
diff -p -u -r1.39 files.h
--- src/files.h 14 May 2005 06:49:47 -0000      1.39
+++ src/files.h 15 Sep 2005 13:30:32 -0000
@@ -24,28 +24,28 @@
 # include "uniqstr.h"
 
 /* File name specified with -o for the output file, or 0 if no -o.  */
-extern char *spec_outfile;
+extern char const *spec_outfile;
 
 /* File name for the parser (i.e., the one above, or its default.) */
-extern char *parser_file_name;
+extern char const *parser_file_name;
 
 /* Symbol prefix specified with -p, or 0 if no -p.  */
 extern const char *spec_name_prefix;
 
 /* File name prefix specified with -b, or 0 if no -b.  */
-extern char *spec_file_prefix;
+extern char const *spec_file_prefix;
 
 /* --verbose. */
-extern char *spec_verbose_file;
+extern char const *spec_verbose_file;
 
 /* File name specified for the output VCG graph.  */
-extern char *spec_graph_file;
+extern char const *spec_graph_file;
 
 /* File name specified with --defines.  */
-extern char *spec_defines_file;
+extern char const *spec_defines_file;
 
-/* Read grammar specifications. */
-extern FILE *finput;
+/* Directory prefix of output file names.  */
+extern char const *dir_prefix;
 
 
 /* If semantic parser, output a .h file that defines YYSTYPE... */
@@ -64,8 +64,5 @@ void compute_output_file_names (void);
 
 FILE *xfopen (const char *name, const char *mode);
 void xfclose (FILE *ptr);
-
-/* Prefix used to generate output file names.  */
-extern char *short_base_name;
 
 #endif /* !FILES_H_ */
Index: src/reader.c
===================================================================
RCS file: /cvsroot/bison/bison/src/reader.c,v
retrieving revision 1.238
diff -p -u -r1.238 reader.c
--- src/reader.c        24 Jul 2005 07:24:22 -0000      1.238
+++ src/reader.c        15 Sep 2005 13:30:32 -0000
@@ -474,8 +474,7 @@ reader (void)
   obstack_init (&pre_prologue_obstack);
   obstack_init (&post_prologue_obstack);
 
-  finput = xfopen (grammar_file, "r");
-  gram_in = finput;
+  gram_in = xfopen (grammar_file, "r");
 
   gram__flex_debug = trace_flag & trace_scan;
   gram_debug = trace_flag & trace_parse;
@@ -523,7 +522,7 @@ reader (void)
   if (! (nsyms <= SYMBOL_NUMBER_MAXIMUM && nsyms == ntokens + nvars))
     abort ();
 
-  xfclose (finput);
+  xfclose (gram_in);
 
   /* Assign the symbols their symbol numbers.  Write #defines for the
      token symbols into FDEFINES if requested.  */
Index: src/scan-skel.l
===================================================================
RCS file: /cvsroot/bison/bison/src/scan-skel.l,v
retrieving revision 1.33
diff -p -u -r1.33 scan-skel.l
--- src/scan-skel.l     9 Sep 2005 22:42:15 -0000       1.33
+++ src/scan-skel.l     15 Sep 2005 13:30:32 -0000
@@ -74,6 +74,7 @@
 
 "@oline@"  fprintf (yyout, "%d", lineno + 1);
 "@ofile@"  QPUTS (outname);
+"@dir_prefix@" QPUTS (dir_prefix);
 "@output_parser_name@" QPUTS (parser_file_name);
 "@output_header_name@" QPUTS (spec_defines_file);
 




reply via email to

[Prev in Thread] Current Thread [Next in Thread]
Index: src/files.c
===================================================================
RCS file: /cvsroot/bison/bison/src/files.c,v
retrieving revision 1.92
diff -p -u -r1.92 files.c
--- src/files.c 9 Sep 2005 22:42:15 -0000       1.92
+++ src/files.c 15 Sep 2005 13:30:32 -0000
@@ -29,16 +29,12 @@
 #include <xstrndup.h>
 
 #include "complain.h"
+#include "dirname.h"
 #include "files.h"
 #include "getargs.h"
 #include "gram.h"
 #include "stdio-safer.h"
 
-/* From basename.c.  Almost a lie, as it returns a char *. */
-const char *base_name (char const *name);
-
-FILE *finput = NULL;
-
 struct obstack pre_prologue_obstack;
 struct obstack post_prologue_obstack;
 
@@ -49,21 +45,36 @@ struct obstack post_prologue_obstack;
    NULL), then the %directive is ignored.  As a result, %name-prefix,
    for instance, will not be honored.  */
 
-char *spec_outfile = NULL;     /* for -o. */
-char *spec_file_prefix = NULL; /* for -b. */
-const char *spec_name_prefix = NULL;   /* for -p. */
-char *spec_verbose_file = NULL;   /* for --verbose. */
-char *spec_graph_file = NULL;   /* for -g. */
-char *spec_defines_file = NULL; /* for --defines. */
-char *parser_file_name = NULL;
+char const *spec_outfile = NULL;       /* for -o. */
+char const *spec_file_prefix = NULL;   /* for -b. */
+char const *spec_name_prefix = NULL;   /* for -p. */
+char const *spec_verbose_file = NULL;  /* for --verbose. */
+char const *spec_graph_file = NULL;    /* for -g. */
+char const *spec_defines_file = NULL;  /* for --defines. */
+char const *parser_file_name;
 
 uniqstr grammar_file = NULL;
 uniqstr current_file = NULL;
 
-static char *full_base_name = NULL;
+/* If --output=dir/foo.c was specified,
+   DIR_PREFIX is `dir/' and ALL_BUT_EXT and ALL_BUT_TAB_EXT are `dir/foo'.
+
+   If --output=dir/foo.tab.c was specified, DIR_PREFIX is `dir/',
+   ALL_BUT_EXT is `dir/foo.tab', and ALL_BUT_TAB_EXT is `dir/foo'.
+
+   If --output was not specified but --file-prefix=dir/foo was specified,
+   ALL_BUT_EXT = `foo.tab' and ALL_BUT_TAB_EXT = `foo'.
+
+   If neither --output nor --file was specified but the input grammar
+   is name dir/foo.y, ALL_BUT_EXT and ALL_BUT_TAB_EXT are `foo'.
 
-/* Prefix used to generate output file names.  */
-char *short_base_name = NULL;
+   If neither --output nor --file was specified, DIR_PREFIX is the
+   empty string (meaning the current directory); otherwise it is
+   `dir/'.  */
+
+static char const *all_but_ext;
+static char const *all_but_tab_ext;
+char const *dir_prefix;
 
 /* C source file extension (the parser source).  */
 static char const *src_extension = NULL;
@@ -121,9 +132,9 @@ xfclose (FILE *ptr)
 }
 
 
-/*---------------------------------------------------------------------.
-| Compute FULL_BASE_NAME, SHORT_BASE_NAME and output files extensions. |
-`---------------------------------------------------------------------*/
+/*------------------------------------------------------------------.
+| Compute ALL_BUT_EXT, ALL_BUT_TAB_EXT and output files extensions. |
+`------------------------------------------------------------------*/
 
 /* Replace all characters FROM by TO in the string IN.
    and returns a new allocated string.  */
@@ -218,29 +229,27 @@ file_name_split (const char *file_name,
 
 
 static void
-compute_base_names (void)
+compute_file_name_parts (void)
 {
   const char *base, *tab, *ext;
 
-  /* If --output=foo.c was specified (SPEC_OUTFILE == foo.c),
-     FULL_BASE_NAME and SHORT_BASE_NAME are `foo'.
-
-     If --output=foo.tab.c was specified, FULL_BASE_NAME is `foo.tab'
-     and SHORT_BASE_NAME is `foo'.
+  /* Compute ALL_BUT_EXT and ALL_BUT_TAB_EXT from SPEC_OUTFILE
+     or GRAMMAR_FILE.
 
      The precise -o name will be used for FTABLE.  For other output
      files, remove the ".c" or ".tab.c" suffix.  */
   if (spec_outfile)
     {
       file_name_split (spec_outfile, &base, &tab, &ext);
+      dir_prefix = xstrndup (spec_outfile, base - spec_outfile);
 
-      /* The full base name goes up the EXT, excluding it. */
-      full_base_name =
+      /* ALL_BUT_EXT goes up the EXT, excluding it. */
+      all_but_ext =
        xstrndup (spec_outfile,
                  (strlen (spec_outfile) - (ext ? strlen (ext) : 0)));
 
-      /* The short base name goes up to TAB, excluding it.  */
-      short_base_name =
+      /* ALL_BUT_TAB_EXT goes up to TAB, excluding it.  */
+      all_but_tab_ext =
        xstrndup (spec_outfile,
                  (strlen (spec_outfile)
                   - (tab ? strlen (tab) : (ext ? strlen (ext) : 0))));
@@ -248,40 +257,36 @@ compute_base_names (void)
       if (ext)
        compute_exts_from_src (ext);
     }
-
-  /* If --file-prefix=foo was specified, FULL_BASE_NAME = `foo.tab'
-     and SHORT_BASE_NAME = `foo'.
-
-     Construct names from it.  */
   else
     {
+      filename_split (grammar_file, &base, &tab, &ext);
+
       if (spec_file_prefix)
        {
          /* If --file-prefix=foo was specified, SHORT_BASE_NAME =
             `foo'.  */
-         short_base_name = xstrdup (spec_file_prefix);
+         dir_prefix = xstrndup (grammar_file, base - grammar_file);
+         all_but_tab_ext = xstrdup (spec_file_prefix);
        }
       else if (yacc_flag)
        {
          /* If --yacc, then the output is `y.tab.c'. */
-         short_base_name = xstrdup ("y");
+         dir_prefix = "";
+         all_but_tab_ext = "y";
        }
       else
        {
          /* Otherwise, the short base name is computed from the input
             grammar: `foo/bar.yy' => `bar'.  */
-         file_name_split (grammar_file, &base, &tab, &ext);
-         short_base_name =
+         dir_prefix = "";
+         all_but_tab_ext =
            xstrndup (base,
                      (strlen (base) - (ext ? strlen (ext) : 0)));
        }
 
-      full_base_name = xmalloc (strlen (short_base_name)
-                               + strlen (TAB_EXT) + 1);
-      stpcpy (stpcpy (full_base_name, short_base_name), TAB_EXT);
+      all_but_ext = concat2 (all_but_tab_ext, TAB_EXT);
 
       /* Compute the extensions from the grammar file name.  */
-      file_name_split (grammar_file, &base, &tab, &ext);
       if (ext && !yacc_flag)
        compute_exts_from_gf (ext);
     }
@@ -299,7 +304,7 @@ compute_output_file_names (void)
   int j;
   int names = 0;
 
-  compute_base_names ();
+  compute_file_name_parts ();
 
   /* If not yet done. */
   if (!src_extension)
@@ -308,25 +313,25 @@ compute_output_file_names (void)
     header_extension = ".h";
 
   name[names++] = parser_file_name =
-    spec_outfile ? spec_outfile : concat2 (full_base_name, src_extension);
+    spec_outfile ? spec_outfile : concat2 (all_but_ext, src_extension);
 
   if (defines_flag)
     {
       if (! spec_defines_file)
-       spec_defines_file = concat2 (full_base_name, header_extension);
+       spec_defines_file = concat2 (all_but_ext, header_extension);
       name[names++] = spec_defines_file;
     }
 
   if (graph_flag)
     {
       if (! spec_graph_file)
-       spec_graph_file = concat2 (short_base_name, ".vcg");
+       spec_graph_file = concat2 (all_but_tab_ext, ".vcg");
       name[names++] = spec_graph_file;
     }
 
   if (report_flag)
     {
-      spec_verbose_file = concat2 (short_base_name, OUTPUT_EXT);
+      spec_verbose_file = concat2 (all_but_tab_ext, OUTPUT_EXT);
       name[names++] = spec_verbose_file;
     }
 
@@ -248,40 +257,36 @@ compute_base_names (void)
       if (spec_file_prefix)
        {
-         /* If --file-prefix=foo was specified, SHORT_BASE_NAME =
+         /* If --file-prefix=foo was specified, ALL_BUT_TAB_EXT =
             `foo'.  */
@@ -248,40 +257,36 @@ compute_base_names (void)
       else
        {
-         /* Otherwise, the short base name is computed from the input
+         /* Otherwise, ALL_BUT_TAB_EXT is computed from the input
             grammar: `foo/bar.yy' => `bar'.  */